@@ -1,29 +1,20 @@
-/**
- @file cryptoUtils.h
- 
- @brief Prototypes for security related functions
- - Key Derivation Function
- - CRC
- - Base32
- - Key agreement algorithm negociation
-
- @author Johan Pascal
-
- @copyright Copyright (C) 2014 Belledonne Communications, Grenoble, France
- 
- This program is free software; you can redistribute it and/or
- modify it under the terms of the GNU General Public License
- as published by the Free Software Foundation; either version 2
- of the License, or (at your option) any later version.
- 
- This program is distributed in the hope that it will be useful,
- but WITHOUT ANY WARRANTY; without even the implied warranty of
- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- GNU General Public License for more details.
- 
- You should have received a copy of the GNU General Public License
- along with this program; if not, write to the Free Software
- Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+/*
+ * Copyright (c) 2014-2019 Belledonne Communications SARL.
+ *
+ * This file is part of bzrtp.
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 #ifndef CRYPTOUTILS_H
 #define CRYPTOUTILS_H
@@ -65,11 +56,11 @@ uint8_t bzrtpUtils_getAvailableCryptoTypes(uint8_t algoType, uint8_t availableTy
  *
  * @return		0 on succes, error code otherwise
  */
-BZRTP_EXPORT int bzrtp_keyDerivationFunction(uint8_t *key, uint16_t keyLength,
-		uint8_t *label, uint16_t labelLength,
-		uint8_t *context, uint16_t contextLength,
-		uint16_t hmacLength,
-		void (*hmacFunction)(uint8_t *, uint8_t, uint8_t *, uint32_t, uint8_t, uint8_t *),
+BZRTP_EXPORT int bzrtp_keyDerivationFunction(const uint8_t *key, const size_t keyLength,
+		const uint8_t *label, const size_t labelLength,
+		const uint8_t *context, const size_t contextLength,
+		const uint16_t hmacLength,
+		void (*hmacFunction)(const uint8_t *, size_t, const uint8_t *, size_t, uint8_t, uint8_t *),
 		uint8_t *output);
 
 
