@@ -1,23 +1,20 @@
-/**
- @file typedef.h
-
- @author Johan Pascal
-
- @copyright Copyright (C) 2014 Belledonne Communications, Grenoble, France
-
- This program is free software; you can redistribute it and/or
- modify it under the terms of the GNU General Public License
- as published by the Free Software Foundation; either version 2
- of the License, or (at your option) any later version.
-
- This program is distributed in the hope that it will be useful,
- but WITHOUT ANY WARRANTY; without even the implied warranty of
- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- GNU General Public License for more details.
-
- You should have received a copy of the GNU General Public License
- along with this program; if not, write to the Free Software
- Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+/*
+ * Copyright (c) 2014-2019 Belledonne Communications SARL.
+ *
+ * This file is part of bzrtp.
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 #ifndef TYPEDEF_H
 #define TYPEDEF_H
@@ -48,11 +45,12 @@
 typedef struct bzrtpChannelContext_struct bzrtpChannelContext_t;
 
 #include <bctoolbox/crypto.h>
+#include <bctoolbox/port.h>
 #include "packetParser.h"
 #include "stateMachine.h"
 
 /* logging */
-#define BCTBX_LOG_DOMAIN "bzrtp"
+/* log domain is defined in CMakeList */
 #include "bctoolbox/logging.h"
 
 #ifdef _WIN32
@@ -160,15 +158,15 @@ struct bzrtpChannelContext_struct {
 	uint16_t selfSequenceNumber; /**< Sequence number of the next packet to be sent */
 	uint16_t peerSequenceNumber; /**< Sequence number of the last valid received packet */
 
-	/* algorithm agreed after Hello message exchange(use mapping define in cryptoWrapper.h) and the function pointer to use them */
-	uint8_t hashAlgo; /**< hash algorithm agreed on after Hello packet exchange, stored using integer mapping defined in cryptoWrapper.h */
+	/* algorithm agreed after Hello message exchange(use mapping define in cryptoUtils.h) and the function pointer to use them */
+	uint8_t hashAlgo; /**< hash algorithm agreed on after Hello packet exchange, stored using integer mapping defined in cryptoUtils.h */
 	uint8_t hashLength; /**< the length in bytes of a hash generated with the agreed hash algo */
-	uint8_t cipherAlgo; /**< cipher algorithm agreed on after Hello packet exchange, stored using integer mapping defined in cryptoWrapper.h */
+	uint8_t cipherAlgo; /**< cipher algorithm agreed on after Hello packet exchange, stored using integer mapping defined in cryptoUtils.h */
 	uint8_t cipherKeyLength; /**< the length in bytes of the key needed by the agreed cipher block algo */
-	uint8_t authTagAlgo; /**< srtp authentication tag algorithm agreed on after Hello packet exchange, stored using integer mapping defined in cryptoWrapper.h */
-	uint8_t keyAgreementAlgo; /**< key agreement algorithm agreed on after Hello packet exchange, stored using integer mapping defined in cryptoWrapper.h */
+	uint8_t authTagAlgo; /**< srtp authentication tag algorithm agreed on after Hello packet exchange, stored using integer mapping defined in cryptoUtils.h */
+	uint8_t keyAgreementAlgo; /**< key agreement algorithm agreed on after Hello packet exchange, stored using integer mapping defined in cryptoUtils.h */
 	uint16_t keyAgreementLength; /**< the length in byte of the secret generated by the agreed key exchange algo */
-	uint8_t sasAlgo; /**< sas rendering algorithm agreed on after Hello packet exchange, stored using integer mapping defined in cryptoWrapper.h */
+	uint8_t sasAlgo; /**< sas rendering algorithm agreed on after Hello packet exchange, stored using integer mapping defined in cryptoUtils.h */
 	uint8_t sasLength; /**< length of the SAS depends on the algorithm agreed */
 
 	/* function pointer to the agreed algorithms - Note, key agreement manage directly this selection so it is not set here */
@@ -204,7 +202,8 @@ struct bzrtpChannelContext_struct {
 struct bzrtpContext_struct {
 	/* contexts */
 	bctbx_rng_context_t *RNGContext; /**< context for random number generation */
-	bctbx_DHMContext_t *DHMContext; /**< context for the Diffie-Hellman-Merkle operations. Only one DHM computation may be done during a call, so this belongs to the general context and not the channel one */
+	void *keyAgreementContext; /**< context for the key agreement operations. Only one key agreement computation may be done during a call, so this belongs to the general context and not the channel one */
+	uint8_t keyAgreementAlgo; /**< key agreement algorithm agreed on the first channel, the one performing key exchange, stored using integer mapping defined in cryptoUtils.h,  */
 
 	/* flags */
 	uint8_t isInitialised; /**< this flag is set once the context was initialised : self ZID retrieved from cache or generated, used to unlock the creation of addtional channels */
@@ -237,6 +236,7 @@ struct bzrtpContext_struct {
 #else
 	void *zidCache; /**< an empty pointer always set to NULL when cache is disabled **/
 #endif /* ZIDCACHE_ENABLED */
+	bctbx_mutex_t *zidCacheMutex; /**< lock access to the cache if provided **/
 	int zuid; /**< internal id used to address zid cache SIP/ZID pair binding **/
 	char *selfURI; /**< a null terminated string storing the local user URI **/
 	uint8_t selfZID[12]; /**< The ZRTP Identifier of this ZRTP end point - a random if running cache less */
@@ -249,6 +249,11 @@ struct bzrtpContext_struct {
 	uint8_t cacheMismatchFlag; /**< Flag set in case of cache mismatch(detected in DHM mode when DH part packet arrives) */
 	uint8_t peerPVS; /**< used to store value of PVS flag sent by peer in the confirm packet on first channel only, then used to compute the PVS value sent to the application */
 
+	/* transient auxiliary shared secret : in addition to the auxiliary shared secret stored in ZID cache, caller can provide a shared secret to the zrtp context which will be used for this transaction only */
+	/* both auxiliary secret are used and combined as transientAuxiliarySecret appended to cachedAuxiliarySecret*/
+	uint8_t *transientAuxSecret; /**< an auxiliary secret not stored in cache, provided after context creation and before the main channel is started */
+	size_t transientAuxSecretLength; /**< size of the previous buffer */
+
 	/* keys */
 	uint8_t *ZRTPSess; /**< ZRTP session key as described in rfc section 4.5.2 */
 	uint8_t	ZRTPSessLength; /**< length of ZRTP session key depends on agreed hash algorithm */
