@@ -1,25 +1,20 @@
-/**
- @file bzrtp.c
-
- @brief Public entry points to the ZRTP implementation
- 
- @author Johan Pascal
-
- @copyright Copyright (C) 2014 Belledonne Communications, Grenoble, France
- 
- This program is free software; you can redistribute it and/or
- modify it under the terms of the GNU General Public License
- as published by the Free Software Foundation; either version 2
- of the License, or (at your option) any later version.
- 
- This program is distributed in the hope that it will be useful,
- but WITHOUT ANY WARRANTY; without even the implied warranty of
- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- GNU General Public License for more details.
- 
- You should have received a copy of the GNU General Public License
- along with this program; if not, write to the Free Software
- Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+/*
+ * Copyright (c) 2014-2019 Belledonne Communications SARL.
+ *
+ * This file is part of bzrtp.
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 #include <string.h>
 #include <stdlib.h>
@@ -42,10 +37,10 @@ static bzrtpChannelContext_t *getChannelContext(bzrtpContext_t *zrtpContext, uin
 static uint8_t copyCryptoTypes(uint8_t destination[7], uint8_t source[7], uint8_t size);
 
 /*
- * Create context structure and initialise it 
+ * Create context structure and initialise it
  *
  * @return The ZRTP engine context data
- *                                                                        
+ *
 */
 bzrtpContext_t *bzrtp_createBzrtpContext(void) {
 	int i;
@@ -56,7 +51,8 @@ bzrtpContext_t *bzrtp_createBzrtpContext(void) {
 	/* start the random number generator */
 	context->RNGContext = bctbx_rng_context_new(); /* TODO: give a seed for the RNG? */
 	/* set the DHM context to NULL, it will be created if needed when creating a DHPart packet */
-	context->DHMContext = NULL;
+	context->keyAgreementContext = NULL;
+	context->keyAgreementAlgo = ZRTP_UNSET_ALGO;
 
 	/* set flags */
 	context->isSecure = 0; /* start unsecure */
@@ -64,11 +60,12 @@ bzrtpContext_t *bzrtp_createBzrtpContext(void) {
 	context->isInitialised = 0; /* will be set by bzrtp_initBzrtpContext */
 
 	/* set to NULL all callbacks pointer */
+	context->zrtpCallbacks.bzrtp_statusMessage = NULL;
 	context->zrtpCallbacks.bzrtp_sendData = NULL;
 	context->zrtpCallbacks.bzrtp_srtpSecretsAvailable = NULL;
 	context->zrtpCallbacks.bzrtp_startSrtpSession = NULL;
 	context->zrtpCallbacks.bzrtp_contextReadyForExportedKeys = NULL;
-	
+
 
 
 	for (i=1; i<ZRTP_MAX_CHANNEL_NUMBER; i++) {
@@ -85,6 +82,7 @@ bzrtpContext_t *bzrtp_createBzrtpContext(void) {
 
 	/* initialise cached secret buffer to null */
 	context->zidCache = NULL; /* a pointer to the sqlite3 db accessor, can be NULL if running cacheless */
+	context->zidCacheMutex = NULL; /* a pointer to a mutex provided by the environment to lock database during access, ignored if NULL */
 	context->zuid = 0;
 	context->peerBzrtpVersion = 0;
 	context->selfURI = NULL;
@@ -98,7 +96,11 @@ bzrtpContext_t *bzrtp_createBzrtpContext(void) {
 	context->cachedSecret.auxsecretLength = 0;
 	context->cacheMismatchFlag = 0;
 	context->peerPVS = 0;
-	
+
+	/* initialise transient shared auxiliary secret buffer */
+	context->transientAuxSecret = NULL;
+	context->transientAuxSecretLength = 0;
+
 	/* initialise key buffers */
 	context->ZRTPSess = NULL;
 	context->ZRTPSessLength = 0;
@@ -112,6 +114,8 @@ bzrtpContext_t *bzrtp_createBzrtpContext(void) {
  * @brief Set the pointer allowing cache access
  *
  * @param[in]	zidCachePointer		Used by internal function to access cache: turn into a sqlite3 pointer if cache is enabled
+ * @param[in]   selfURI			Local URI used for this communication, needed to perform cache operation, NULL terminated string, duplicated by this function
+ * @param[in]   peerURI			Peer URI used for this communication, needed to perform cache operation, NULL terminated string, duplicated by this function
  *
  * @return 0 or BZRTP_CACHE_SETUP(if cache is populated by this call) on success, error code otherwise
 */
@@ -124,11 +128,47 @@ int bzrtp_setZIDCache(bzrtpContext_t *context, void *zidCache, const char *selfU
 
 	/* zidCache pointer is actually a pointer to sqlite3 db, store it in context */
 	context->zidCache = (sqlite3 *)zidCache;
+	if (context->selfURI != NULL) {
+		free(context->selfURI);
+	}
 	context->selfURI = strdup(selfURI);
+
+	if (context->peerURI != NULL) {
+		free(context->peerURI);
+	}
 	context->peerURI = strdup(peerURI);
 
 	/* and init the cache(create needed tables if they don't exist) */
-	return bzrtp_initCache(context->zidCache);
+	return bzrtp_initCache_lock(context->zidCache, context->zidCacheMutex);
+#else /* ZIDCACHE_ENABLED */
+	return BZRTP_ERROR_CACHEDISABLED;
+#endif /* ZIDCACHE_ENABLED */
+}
+
+/**
+ * @brief Set the pointer allowing cache access, this version of the function get a mutex to lock the cache when accessing it
+ *
+ * Note: bzrtp does not manage the given mutex pointer, it has to be already initialized
+ *       and shall be destroyed by environment after the BZRTP session is completed.
+ *
+ * @param[in]	zidCachePointer		Used by internal function to access cache: turn into a sqlite3 pointer if cache is enabled
+ * @param[in]   selfURI			Local URI used for this communication, needed to perform cache operation, NULL terminated string, duplicated by this function
+ * @param[in]   peerURI			Peer URI used for this communication, needed to perform cache operation, NULL terminated string, duplicated by this function
+ * @param[in]   zidCacheMutex		Points to a mutex used to lock zidCache database access. Can be NULL, the zidcache is then non locked.
+ *
+ * @return 0 or BZRTP_CACHE_SETUP(if cache is populated by this call) on success, error code otherwise
+*/
+int bzrtp_setZIDCache_lock(bzrtpContext_t *context, void *zidCache, const char *selfURI, const char *peerURI, bctbx_mutex_t *zidCacheMutex) {
+#ifdef ZIDCACHE_ENABLED
+	/* is zrtp context valid */
+	if (context==NULL) {
+		return BZRTP_ERROR_INVALIDCONTEXT;
+	}
+
+	context->zidCacheMutex = zidCacheMutex;
+
+	/* have the non lockable function finish the job */
+	return  bzrtp_setZIDCache(context, zidCache, selfURI, peerURI);
 #else /* ZIDCACHE_ENABLED */
 	return BZRTP_ERROR_CACHEDISABLED;
 #endif /* ZIDCACHE_ENABLED */
@@ -153,7 +193,7 @@ int bzrtp_initBzrtpContext(bzrtpContext_t *context, uint32_t selfSSRC) {
 	}
 
 	/* initialise self ZID. Randomly generated if no ZID is found in cache or no cache found */
-	bzrtp_getSelfZID(context->zidCache, context->selfURI, context->selfZID, context->RNGContext);
+	bzrtp_getSelfZID_lock(context->zidCache, context->selfURI, context->selfZID, context->RNGContext, context->zidCacheMutex);
 	context->isInitialised = 1;
 
 	/* allocate 1 channel context, set all the others pointers to NULL */
@@ -168,7 +208,7 @@ int bzrtp_initBzrtpContext(bzrtpContext_t *context, uint32_t selfSSRC) {
  * @param[in]	selfSSRC	The SSRC identifying the channel to be destroyed
  *
  * @return the number of channel still active in this ZRTP context
- *                                                                           
+ *
 */
 int bzrtp_destroyBzrtpContext(bzrtpContext_t *context, uint32_t selfSSRC) {
 	int i;
@@ -196,10 +236,16 @@ int bzrtp_destroyBzrtpContext(bzrtpContext_t *context, uint32_t selfSSRC) {
 
 
 	/* We have no more channel, destroy the zrtp context */
-	/* DHM context shall already been destroyed after s0 computation, but just in case */
-	if (context->DHMContext != NULL) {
-		bctbx_DestroyDHMContext(context->DHMContext);
-		context->DHMContext = NULL;
+	/* key agreement context shall already been destroyed after s0 computation, but just in case */
+	if (context->keyAgreementContext != NULL) {
+		if (context->keyAgreementAlgo == ZRTP_KEYAGREEMENT_X255 || context->keyAgreementAlgo == ZRTP_KEYAGREEMENT_X448) {
+			bctbx_DestroyECDHContext((bctbx_ECDHContext_t *)context->keyAgreementContext);
+		}
+		if (context->keyAgreementAlgo == ZRTP_KEYAGREEMENT_DH2k || context->keyAgreementAlgo == ZRTP_KEYAGREEMENT_DH3k) {
+			bctbx_DestroyDHMContext((bctbx_DHMContext_t *)context->keyAgreementContext);
+		}
+		context->keyAgreementContext = NULL;
+		context->keyAgreementAlgo = ZRTP_UNSET_ALGO;
 	}
 
 	/* Destroy keys and secrets */
@@ -239,7 +285,14 @@ int bzrtp_destroyBzrtpContext(bzrtpContext_t *context, uint32_t selfSSRC) {
 
 	free(context->selfURI);
 	free(context->peerURI);
-	
+
+	/* transient shared auxiliary secret */
+	if (context->transientAuxSecret != NULL) {
+		bzrtp_DestroyKey(context->transientAuxSecret, context->transientAuxSecretLength, context->RNGContext);
+		free(context->transientAuxSecret);
+		context->transientAuxSecret=NULL;
+	}
+
 	/* destroy the RNG context at the end because it may be needed to destroy some keys */
 	bctbx_rng_context_free(context->RNGContext);
 	context->RNGContext = NULL;
@@ -418,7 +471,7 @@ int bzrtp_iterate(bzrtpContext_t *zrtpContext, uint32_t selfSSRC, uint64_t timeR
  * @param[in]		clientData		The clientData pointer, casted to a (void *)
  *
  * @return 0 on success
- *                                                                           
+ *
 */
 int bzrtp_setClientData(bzrtpContext_t *zrtpContext, uint32_t selfSSRC, void *clientData) {
 	/* get channel context */
@@ -489,7 +542,7 @@ int bzrtp_processMessage(bzrtpContext_t *zrtpContext, uint32_t selfSSRC, uint8_t
 		bzrtp_freeZrtpPacket(zrtpPacket);
 		bzrtp_freeZrtpPacket(pingAckPacket);
 		zrtpChannelContext->pingPacket = NULL;
-		
+
 		return retval;
 	}
 
@@ -513,7 +566,7 @@ int bzrtp_processMessage(bzrtpContext_t *zrtpContext, uint32_t selfSSRC, uint8_t
 void bzrtp_SASVerified(bzrtpContext_t *zrtpContext) {
 	if (zrtpContext != NULL) {
 		uint8_t pvsFlag = 1;
-		char *colNames[] = {"pvs"};
+		const char *colNames[] = {"pvs"};
 		uint8_t *colValues[] = {&pvsFlag};
 		size_t colLength[] = {1};
 
@@ -522,7 +575,7 @@ void bzrtp_SASVerified(bzrtpContext_t *zrtpContext) {
 			zrtpContext->cacheMismatchFlag  = 0;
 			bzrtp_updateCachedSecrets(zrtpContext, zrtpContext->channelContext[0]); /* channel[0] is the only one in DHM mode, so the only one able to have a cache mismatch */
 		}
-		bzrtp_cache_write(zrtpContext->zidCache, zrtpContext->zuid, "zrtp", colNames, colValues, colLength, 1);
+		bzrtp_cache_write_lock(zrtpContext->zidCache, zrtpContext->zuid, "zrtp", colNames, colValues, colLength, 1, zrtpContext->zidCacheMutex);
 	}
 }
 
@@ -535,14 +588,13 @@ void bzrtp_SASVerified(bzrtpContext_t *zrtpContext) {
 void bzrtp_resetSASVerified(bzrtpContext_t *zrtpContext) {
 	if (zrtpContext != NULL) {
 		uint8_t pvsFlag = 0;
-		char *colNames[] = {"pvs"};
+		const char *colNames[] = {"pvs"};
 		uint8_t *colValues[] = {&pvsFlag};
 		size_t colLength[] = {1};
-		bzrtp_cache_write(zrtpContext->zidCache, zrtpContext->zuid, "zrtp", colNames, colValues, colLength, 1);
+		bzrtp_cache_write_lock(zrtpContext->zidCache, zrtpContext->zuid, "zrtp", colNames, colValues, colLength, 1, zrtpContext->zidCacheMutex);
 	}
 }
 
-
 /*
  * @brief  Allow client to compute an exported according to RFC section 4.5.2
  *		Check the context is ready(we already have a master exported key and KDF context)
@@ -560,7 +612,8 @@ int bzrtp_exportKey(bzrtpContext_t *zrtpContext, char *label, size_t labelLength
 	/* check we have s0 or exportedKey and KDFContext in channel[0] - export keys is available only on channel 0 completion - see RFC 4.5.2 */
 	bzrtpChannelContext_t *zrtpChannelContext = zrtpContext->channelContext[0];
 
-	if (zrtpContext->peerBzrtpVersion == 10000) { /* keep compatibility with older implementation of bzrtp */
+	if (zrtpContext->peerBzrtpVersion == 0x010000) { /* keep compatibility with older implementation of bzrtp */
+#ifdef SUPPORT_EXPORTEDKEY_V010000
 		/* before version 1.1.0 (turned into an int MMmmpp -> 010100) exported keys wrongly derives from given label and s0 direclty instead of
 		deriving one Exported Key from S0 and then as many as needed from the exported key as specified in the RFC section 4.5.2 */
 		if (zrtpChannelContext->s0 == NULL || zrtpChannelContext->KDFContext == NULL) {
@@ -572,8 +625,13 @@ int bzrtp_exportKey(bzrtpContext_t *zrtpContext, char *label, size_t labelLength
 			*derivedKeyLength = zrtpChannelContext->hashLength;
 		}
 
-		bzrtp_keyDerivationFunction(zrtpChannelContext->s0, zrtpChannelContext->hashLength, (uint8_t *)label, labelLength, zrtpChannelContext->KDFContext, zrtpChannelContext->KDFContextLength, *derivedKeyLength, (void (*)(uint8_t *, uint8_t,  uint8_t *, uint32_t,  uint8_t,  uint8_t *))zrtpChannelContext->hmacFunction, derivedKey);
-
+		bzrtp_keyDerivationFunction(zrtpChannelContext->s0, zrtpChannelContext->hashLength, (uint8_t *)label, labelLength, zrtpChannelContext->KDFContext, zrtpChannelContext->KDFContextLength, *derivedKeyLength, zrtpChannelContext->hmacFunction, derivedKey);
+#else /* SUPPORT_EXPORTEDKEY_V010000 */
+		/* We do not support anymore backward compatibility, just do nothing but send an error message*/
+		if (zrtpContext->zrtpCallbacks.bzrtp_statusMessage!=NULL && zrtpContext->zrtpCallbacks.bzrtp_messageLevel>=BZRTP_MESSAGE_ERROR) { /* use error level as we explicitely compile with no support for older version */
+				zrtpContext->zrtpCallbacks.bzrtp_statusMessage(zrtpChannelContext->clientData, BZRTP_MESSAGE_ERROR, BZRTP_MESSAGE_PEERVERSIONOBSOLETE, "obsolete bzrtp version are not supported anymore");
+		}
+#endif /* SUPPORT_EXPORTEDKEY_V010000 */
 	} else { /* peer either use version 1.1 of BZRTP or another library, just stick to the RFC to create the export key */
 		if ((zrtpChannelContext->s0 == NULL && zrtpContext->exportedKey) || zrtpChannelContext->KDFContext == NULL) {
 			return BZRTP_ERROR_INVALIDCONTEXT;
@@ -583,7 +641,7 @@ int bzrtp_exportKey(bzrtpContext_t *zrtpContext, char *label, size_t labelLength
 		if (zrtpContext->exportedKey == NULL) {
 			zrtpContext->exportedKeyLength = zrtpChannelContext->hashLength;
 			zrtpContext->exportedKey = (uint8_t *)malloc(zrtpContext->exportedKeyLength*sizeof(uint8_t));
-			bzrtp_keyDerivationFunction(zrtpChannelContext->s0, zrtpChannelContext->hashLength, (uint8_t *)"Exported key", 12, zrtpChannelContext->KDFContext, zrtpChannelContext->KDFContextLength, zrtpContext->exportedKeyLength, (void (*)(uint8_t *, uint8_t,  uint8_t *, uint32_t,  uint8_t,  uint8_t *))zrtpChannelContext->hmacFunction, zrtpContext->exportedKey);
+			bzrtp_keyDerivationFunction(zrtpChannelContext->s0, zrtpChannelContext->hashLength, (uint8_t *)"Exported key", 12, zrtpChannelContext->KDFContext, zrtpChannelContext->KDFContextLength, zrtpContext->exportedKeyLength, zrtpChannelContext->hmacFunction, zrtpContext->exportedKey);
 		}
 
 		/* We derive a maximum of hashLength bytes */
@@ -591,7 +649,7 @@ int bzrtp_exportKey(bzrtpContext_t *zrtpContext, char *label, size_t labelLength
 			*derivedKeyLength = zrtpChannelContext->hashLength;
 		}
 
-		bzrtp_keyDerivationFunction(zrtpContext->exportedKey, zrtpChannelContext->hashLength, (uint8_t *)label, labelLength, zrtpChannelContext->KDFContext, zrtpChannelContext->KDFContextLength, *derivedKeyLength, (void (*)(uint8_t *, uint8_t,  uint8_t *, uint32_t,  uint8_t,  uint8_t *))zrtpChannelContext->hmacFunction, derivedKey);
+		bzrtp_keyDerivationFunction(zrtpContext->exportedKey, zrtpChannelContext->hashLength, (uint8_t *)label, labelLength, zrtpChannelContext->KDFContext, zrtpChannelContext->KDFContextLength, *derivedKeyLength, zrtpChannelContext->hmacFunction, derivedKey);
 
 	}
 	return 0;
@@ -901,6 +959,51 @@ int bzrtp_getSelfHelloHash(bzrtpContext_t *zrtpContext, uint32_t selfSSRC, uint8
 	return 0;
 }
 
+/**
+ * @brief Set Auxiliary Secret for this channel(shall be used only on primary audio channel)
+ *   The given auxSecret is appended to any aux secret found in ZIDcache
+ *   This function must be called before reception of peerHello packet
+ *
+ * @param[in]		zrtpContext	The ZRTP context we're dealing with
+ * @param[in]		auxSecret	A buffer holding the auxiliary shared secret to use (see RFC 6189 section 4.3)
+ * @param[in]		auxSecretLength	lenght of the previous buffer
+ *
+ * @return 0 on success, error code otherwise
+ *
+ * @note The auxiliary shared secret mechanic is used by LIMEv2 for encryption security purposes but might be used for its original purpose in a regular
+ * ZRTP session if it becomes necessary in the future, or by another encryption engine for example. In that case the API will need an adaptation work.
+ */
+int bzrtp_setAuxiliarySharedSecret(bzrtpContext_t *zrtpContext, const uint8_t *auxSecret, size_t auxSecretLength) {
+	if (zrtpContext == NULL) {
+		return BZRTP_ERROR_INVALIDCONTEXT;
+	}
+
+	if (zrtpContext->channelContext[0] && zrtpContext->channelContext[0]->peerPackets[HELLO_MESSAGE_STORE_ID] != NULL) {
+		return BZRTP_ERROR_CONTEXTNOTREADY;
+	}
+
+	/* allocate memory to store the secret - check it wasn't already allocated */
+	if (zrtpContext->transientAuxSecret) {
+		free(zrtpContext->transientAuxSecret);
+	}
+	zrtpContext->transientAuxSecret = (uint8_t *)malloc(auxSecretLength*sizeof(uint8_t));
+
+	/* copy the aux secret and length */
+	memcpy(zrtpContext->transientAuxSecret, auxSecret, auxSecretLength);
+	zrtpContext->transientAuxSecretLength = auxSecretLength;
+
+	return 0;
+}
+
+/**
+ * @brief Get the ZRTP auxiliary shared secret mismatch status
+ *
+ * @param[in]		zrtpContext			The ZRTP context we're dealing with
+ * @return	BZRTP_AUXSECRET_MATCH on match, BZRTP_AUXSECRET_MISMATCH on mismatch, BZRTP_AUXSECRET_UNSET if auxiliary shared secret is unused
+ */
+uint8_t bzrtp_getAuxiliarySharedSecretMismatch(bzrtpContext_t *zrtpContext) {
+	return zrtpContext->channelContext[0]->srtpSecrets.auxSecretMismatch;
+}
 
 /**
  * @brief Get the channel status
@@ -950,7 +1053,7 @@ static bzrtpChannelContext_t *getChannelContext(bzrtpContext_t *zrtpContext, uin
 	if (zrtpContext==NULL) {
 		return NULL;
 	}
-	
+
 	for (i=0; i<ZRTP_MAX_CHANNEL_NUMBER; i++) {
 		if (zrtpContext->channelContext[i]!=NULL) {
 			if (zrtpContext->channelContext[i]->selfSSRC == selfSSRC) {
@@ -964,7 +1067,7 @@ static bzrtpChannelContext_t *getChannelContext(bzrtpContext_t *zrtpContext, uin
 /**
  * @brief Initialise the context of a channel and create and store the Hello packet
  * Initialise some vectors
- * 
+ *
  * @param[in] 		zrtpContext			The zrtpContext hosting this channel, needed to acces the RNG
  * @param[out]		zrtpChanneContext	The channel context to be initialised
  * @param[in]		selfSSRC			The SSRC allocated to this channel
@@ -1054,6 +1157,7 @@ static int bzrtp_initChannelContext(bzrtpContext_t *zrtpContext, bzrtpChannelCon
 	zrtpChannelContext->srtpSecrets.keyAgreementAlgo = ZRTP_UNSET_ALGO;
 	zrtpChannelContext->srtpSecrets.sasAlgo = ZRTP_UNSET_ALGO;
 	zrtpChannelContext->srtpSecrets.cacheMismatch = 0;
+	zrtpChannelContext->srtpSecrets.auxSecretMismatch = BZRTP_AUXSECRET_UNSET;
 
 	/* create the Hello packet and store it */
 	helloPacket = bzrtp_createZrtpPacket(zrtpContext, zrtpChannelContext, MSGTYPE_HELLO, &retval);
@@ -1074,7 +1178,7 @@ static int bzrtp_initChannelContext(bzrtpContext_t *zrtpContext, bzrtpChannelCon
 /**
  * @brief Destroy the context of a channel
  * Free allocated buffers, destroy keys
- * 
+ *
  * @param[in] 		zrtpContext			The zrtpContext hosting this channel, needed to acces the RNG
  * @param[in] 		zrtpChannelContext	The channel context to be destroyed
  */
