@@ -1,23 +1,20 @@
-/**
- @file testUtils.c
-
- @author Johan Pascal
-
- @copyright Copyright (C) 2014 Belledonne Communications, Grenoble, France
- 
- This program is free software; you can redistribute it and/or
- modify it under the terms of the GNU General Public License
- as published by the Free Software Foundation; either version 2
- of the License, or (at your option) any later version.
- 
- This program is distributed in the hope that it will be useful,
- but WITHOUT ANY WARRANTY; without even the implied warranty of
- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- GNU General Public License for more details.
- 
- You should have received a copy of the GNU General Public License
- along with this program; if not, write to the Free Software
- Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+/*
+ * Copyright (c) 2014-2019 Belledonne Communications SARL.
+ *
+ * This file is part of bzrtp.
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
 
@@ -28,6 +25,8 @@
 #include "testUtils.h"
 #include "cryptoUtils.h"
 
+#include "bzrtpTest.h"
+
 #ifndef _WIN32
 #if !defined(__QNXNTO__) && !(defined(__ANDROID__) && defined(__LP64__))
 #include <ctype.h>
@@ -60,6 +59,8 @@ const char *bzrtp_keyAgreement_toString(uint8_t keyAgreementAlgo) {
 		case(ZRTP_KEYAGREEMENT_DH3k): return "DHM-3072";
 		case(ZRTP_KEYAGREEMENT_EC38): return "ECDH-384";
 		case(ZRTP_KEYAGREEMENT_EC52): return "ECDH-521";
+		case(ZRTP_KEYAGREEMENT_X255): return "X25519";
+		case(ZRTP_KEYAGREEMENT_X448): return "X448";
 		case(ZRTP_KEYAGREEMENT_Prsh): return "PreShared";
 		case(ZRTP_KEYAGREEMENT_Mult): return "MultiStream";
 		default: return "Unknown Algo";
@@ -99,17 +100,6 @@ const char *bzrtp_sas_toString(uint8_t sasAlgo) {
 		}
 }
 
-
-void bzrtp_message(const char *fmt, ...) {
-	if (verbose) {
-		va_list args;
-		va_start(args, fmt);
-		vprintf(fmt, args);
-		va_end(args);
-		fflush(NULL);
-	}
-}
-
 void printHex(char *title, uint8_t *data, uint32_t length) {
 	if (verbose) {
 		uint32_t i;
